@@ -18,7 +18,8 @@ from typing import Dict, List, Optional, Tuple
 
 import numpy as np
 
-from braket.default_simulator.operation import Observable
+from braket.default_simulator import gate_operations
+from braket.default_simulator.operation import GateOperation, Observable
 from braket.default_simulator.operation_helpers import (
     check_hermitian,
     check_matrix_dimensions,
@@ -51,9 +52,8 @@ class Identity(Observable):
     def eigenvalues(self) -> np.ndarray:
         return np.array([1, 1])
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        return None
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        return ()
 
 
 class Hadamard(Observable):
@@ -81,13 +81,14 @@ class Hadamard(Observable):
     def eigenvalues(self) -> np.ndarray:
         return pauli_eigenvalues(1)
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        # RY(-\pi / 4)
-        angle = -math.pi / 4
-        cos_component = math.cos(angle / 2)
-        sin_component = math.sin(angle / 2)
-        return np.array([[cos_component, -sin_component], [sin_component, cos_component]])
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        if self._targets is None:
+            return tuple(Hadamard._diagonalizing_gate([target]) for target in range(num_qubits))
+        return (Hadamard._diagonalizing_gate(self._targets),)
+
+    @staticmethod
+    def _diagonalizing_gate(targets):
+        return gate_operations.RotY(targets, -math.pi / 4)
 
 
 class PauliX(Observable):
@@ -115,10 +116,14 @@ class PauliX(Observable):
     def eigenvalues(self) -> np.ndarray:
         return pauli_eigenvalues(1)
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        # H
-        return np.array([[1, 1], [1, -1]]) / math.sqrt(2)
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        if self._targets is None:
+            return tuple(PauliX._diagonalizing_gate([target]) for target in range(num_qubits))
+        return (PauliX._diagonalizing_gate(self._targets),)
+
+    @staticmethod
+    def _diagonalizing_gate(targets):
+        return gate_operations.Hadamard(targets)
 
 
 class PauliY(Observable):
@@ -131,6 +136,9 @@ class PauliY(Observable):
         operator to evolve the state of the system.
     """
 
+    # HS^{\dagger}
+    _diagonalizing_matrix = np.array([[1, -1j], [1, 1j]]) / math.sqrt(2)
+
     def __init__(self, targets: Optional[List[int]] = None):
         self._targets = _validate_and_clone_single_qubit_target(targets)
 
@@ -146,10 +154,14 @@ class PauliY(Observable):
     def eigenvalues(self) -> np.ndarray:
         return pauli_eigenvalues(1)
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        # HS^{\dagger}
-        return np.array([[1, -1j], [1, 1j]]) / math.sqrt(2)
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        if self._targets is None:
+            return tuple(PauliY._diagonalizing_gate([target]) for target in range(num_qubits))
+        return (PauliY._diagonalizing_gate(self._targets),)
+
+    @staticmethod
+    def _diagonalizing_gate(targets):
+        return gate_operations.Unitary(targets, PauliY._diagonalizing_matrix)
 
 
 class PauliZ(Observable):
@@ -181,9 +193,9 @@ class PauliZ(Observable):
     def eigenvalues(self) -> np.ndarray:
         return pauli_eigenvalues(1)
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        return None
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        # Already diagonalized
+        return ()
 
 
 class Hermitian(Observable):
@@ -203,6 +215,9 @@ class Hermitian(Observable):
             )
         check_hermitian(clone)
         self._matrix = clone
+        eigendecomposition = Hermitian._eigendecomposition(clone)
+        self._eigenvalues = eigendecomposition["eigenvalues"]
+        self._diagonalizing_matrix = eigendecomposition["eigenvectors"].conj().T
 
     @property
     def matrix(self) -> np.ndarray:
@@ -215,28 +230,36 @@ class Hermitian(Observable):
 
     @property
     def eigenvalues(self) -> np.ndarray:
-        return self._eigendecomposition()["eigenvalues"]
+        return self._eigenvalues
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        return self._eigendecomposition()["eigenvectors"].conj().T
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        if self._targets is None:
+            return tuple(self._diagonalizing_gate([target]) for target in range(num_qubits))
+        return (self._diagonalizing_gate(self._targets),)
 
-    def _eigendecomposition(self) -> Dict[str, np.ndarray]:
+    def _diagonalizing_gate(self, targets):
+        return gate_operations.Unitary(targets, self._diagonalizing_matrix)
+
+    @staticmethod
+    def _eigendecomposition(matrix: np.ndarray) -> Dict[str, np.ndarray]:
         """Decomposes the Hermitian matrix into its eigenvectors and associated eigenvalues.
 
         The eigendecomposition is cached so that if another Hermitian observable
         is created with the same matrix, the eigendecomposition doesn't have to
         be recalculated.
 
+        Args:
+            matrix (np.ndarray): The matrix to decompose
+
         Returns:
             Dict[str, np.ndarray]: The keys are "eigenvectors", mapping to a matrix whose
             columns are the eigenvectors of the matrix, and "eigenvalues", a list of
             associated eigenvalues in the order their corresponding eigenvectors in the
             "eigenvectors" matrix
         """
-        mat_key = tuple(self._matrix.flatten().tolist())
+        mat_key = tuple(matrix.flatten().tolist())
         if mat_key not in Hermitian._eigenpairs:
-            eigenvalues, eigenvectors = np.linalg.eigh(self.matrix)
+            eigenvalues, eigenvectors = np.linalg.eigh(matrix)
             Hermitian._eigenpairs[mat_key] = {
                 "eigenvectors": eigenvectors,
                 "eigenvalues": eigenvalues,
@@ -261,9 +284,8 @@ class TensorProduct(Observable):
         self._measured_qubits = tuple(
             qubit for observable in factors for qubit in observable.measured_qubits
         )
-        self._diagonalizing_matrix = TensorProduct._construct_matrix(factors)
         self._eigenvalues = TensorProduct._compute_eigenvalues(factors, self._measured_qubits)
-        self._factors = factors
+        self._factors = tuple(factors)
 
     @property
     def factors(self) -> Tuple[Observable]:
@@ -281,20 +303,8 @@ class TensorProduct(Observable):
     def eigenvalues(self) -> np.ndarray:
         return self._eigenvalues
 
-    @property
-    def diagonalizing_matrix(self) -> Optional[np.ndarray]:
-        return self._diagonalizing_matrix
-
-    @staticmethod
-    def _construct_matrix(factors: List[Observable]) -> Optional[np.ndarray]:
-        matrices = tuple(
-            factor.diagonalizing_matrix
-            for factor in factors
-            # Ignore observables with trivial diagonalizing matrices
-            if factor.diagonalizing_matrix is not None
-        )
-        # (A \otimes I)(I \otimes B) == A \otimes B
-        return functools.reduce(np.kron, matrices) if matrices else None
+    def diagonalizing_gates(self, num_qubits: Optional[int] = None) -> Tuple[GateOperation, ...]:
+        return sum((factor.diagonalizing_gates() for factor in self._factors), ())
 
     @staticmethod
     def _compute_eigenvalues(factors: List[Observable], qubits: Tuple[int, ...]) -> np.ndarray:
