@@ -15,7 +15,7 @@ from typing import List
 
 import numpy as np
 
-from braket.default_simulator.operation import GateOperation, Observable, Operation
+from braket.default_simulator.operation import GateOperation, Observable
 from braket.default_simulator.simulation_strategies import (
     batch_operation_strategy,
     single_operation_strategy,
@@ -94,13 +94,19 @@ class StateVectorSimulation:
         """
         if self._post_observables is not None:
             raise RuntimeError("Observables have already been applied.")
+        operations = list(
+            sum(
+                [observable.diagonalizing_gates(self._qubit_count) for observable in observables],
+                (),
+            )
+        )
         self._post_observables = StateVectorSimulation._apply_operations(
-            self._state_vector, self._qubit_count, observables, self._batch_size
+            self._state_vector, self._qubit_count, operations, self._batch_size
         )
 
     @staticmethod
     def _apply_operations(
-        state: np.ndarray, qubit_count: int, operations: List[Operation], batch_size: int
+        state: np.ndarray, qubit_count: int, operations: List[GateOperation], batch_size: int
     ) -> np.ndarray:
         state_tensor = np.reshape(state, [2] * qubit_count)
         final = (
